pro extract_WC2

; default value for ifilform
ifilform = 0
  
;=========== TO BE CHANGED BY USER ==================================
; ifile = flag to select lecture of coordinates from an input file
; ifile = 0   pixels to be extracted defined by min and max long/lat (rectangular area) 
; ifile = 1   pixels (long, lat) to be extracted defined in an input file
ifile=0

; kwrite = flag to write or not the extracted data in output files
; kwrite = 0  no writing
; kwrite = 1  write outputs
; kwrite = 2  write elevation data only 
kwrite = 1


; output directory where output files will be written
outdir='../clim/Belgium/WC2/'

; kmap = flag to plot maps of the data on the screen
; kmap = 0  no plot
; kmap = 1  plot maps of outputs 
kmap = 1

; elvthr = elevation threshold (m) below which no extraction 
; of data is performed
elvthr = -500.

if ifile eq 0 then begin
 ; extraction of a rectangular area 
 ; provide minimum and maximum longitudes/latitudes

;Belgium
 lgmin = 1.75
 lgmax = 7.0
 ltmin = 49.25
 ltmax = 51.75
;Luxemburg 
; lgmin = 5.7
; lgmax = 6.6  
; ltmin = 49.4 
; ltmax = 50.2
;Morocco
; lgmin = -19.
; lgmax = -0.5  
; ltmin = 21. 
; ltmax = 36.5 
      
 
endif else begin
 ; extraction of a set of pixels defined in an input file 
 ;      (1st col:longitude (deg); 2nd col:latitude (deg)) 
 ; provide the name of the input file and the number of pixels 
 ;      (i.e., lines) in the file 
 ; pix_amplify = amplification factor for the size of the pixel
 ;               when you plot your results on a map (kmap=1)
 ; ifilform = flag to determine the format of input file 
 ; ifilform = -1 nnn,xlg,xlt
 ; ifilform = 0 xlg,xlt  
 ; ifilform = 1 xlg,xlt+species and species file rewritten as an output file
 ; nspecies = number of species considered when ifilform = 1
   npix_max=long(31243) 
   ifilform = -1
   file_in='../clim/Belgium/ecotxt_2017.dat'
   pix_amplify = 1.
   if ifilform eq 1 then nspecies = 7   
endelse


;====================================================================

xlgtile=fltarr(12)
xlttile=fltarr(6) 

for ilg = 0, 11  do xlgtile(ilg)=-180.+(ilg+1)*30.
for ilt = 0, 5  do xlttile(ilt)=-90.+(ilt+1)*30. 
 
exist_tile=fltarr(12,6)
for ilg = 0, 11  do begin 
  for ilt = 0, 5 do exist_tile(ilg,ilt)=0 
endfor
  
close,/all

dlong=1./120.
dlat=1./120.

if ifile eq 0 then begin

  if lgmin le -180 then lgmin = lgmin+360
  if lgmin gt 180 then lgmin = lgmin-360
  if lgmax le -180 then lgmax = lgmax+360
  if lgmax gt 180 then lgmax = lgmax-360
  if lgmin ge lgmax then begin
    print,'Minimum longitude larger than maximum longitude: ',lgmin,lgmax
    goto,finish
  endif
  if ltmin ge ltmax then begin
    print,'Minimum latitude larger than maximum latitude:',ltmin,ltmax
    goto,finish
  endif 
   
 nlong = long((lgmax-lgmin)*120.+0.00001)
 nlat = long((ltmax-ltmin)*120.+0.00001)
; print,nlong,nlat
 ylong = fltarr(nlong)
 ylat = fltarr(nlat)
 for ilong=1,nlong do ylong(ilong-1)=lgmin+dlong*(0.5+float(ilong-1))
 for ilat=1,nlat do ylat(ilat-1)=ltmin+dlat*(0.5+float(ilat-1))

 npix_max = long(nlong*nlat)
 xlong_pix=fltarr(npix_max)
 xlat_pix=fltarr(npix_max) 
 npix = long(0)
 for ilat = 1, nlat do begin
  for ilong = 1, nlong do begin
 ;  print,nlong,nlat,npix_max,npix
   xlong_pix(npix)=ylong(ilong-1)
   xlat_pix(npix)=ylat(ilat-1)
   npix = npix + long(1)
  endfor
 endfor 

endif else begin


 openr,10,file_in
 
 xlong_pix=fltarr(npix_max)
 xlat_pix=fltarr(npix_max)
 
 if ifilform eq 1 then begin 
  ioc_pix=intarr(npix_max,nspecies)
  ioc=intarr(nspecies)
 endif
 for npix = long(0), npix_max-long(1) do begin
  if ifilform eq 1 then begin
    readf,10,xlg,xlt,ioc
  endif else begin
   if ifilform eq -1 then begin 
     readf,10,nnn,xlg,xlt
   endif else begin
     readf,10,xlg,xlt
   endelse
  endelse
  if xlg le -180 then xlg = xlg+360
  if xlg gt 180 then xlg = xlg-360
  xlong_pix(npix)=xlg
  xlat_pix(npix)=xlt
  if ifilform eq 1 then begin
   for isp=0,nspecies-1 do begin
     ioc_pix(npix,isp)=ioc(isp)
   endfor
  endif
 endfor
 border = 1.
 lgmin = fix(min(xlong_pix)-border)  
 lgmax = fix(max(xlong_pix)+border)  
 ltmin = fix(min(xlat_pix)-border) 
 ltmax = fix(max(xlat_pix)+border)
  
endelse

tile_pix=intarr(npix_max,2)
alt_pix=fltarr(npix_max)
tavg_pix=fltarr(npix_max,12)
delt_pix=fltarr(npix_max,12)
prec_pix=fltarr(npix_max,12)
srad_pix=fltarr(npix_max,12)
rhu_pix=fltarr(npix_max,12)
vapr_pix=fltarr(npix_max,12)
wind_pix=fltarr(npix_max,12)

for npix = long(0), npix_max-long(1) do begin
  for ilg = 0, 11  do begin 
    if xlong_pix(npix) lt xlgtile(ilg) then begin
      tile_pix(npix,0)=ilg 
      goto,lg_end 
    endif
  endfor
  lg_end: 
  for ilt = 0, 5  do begin
    if xlat_pix(npix) lt xlttile(ilt) then begin
      tile_pix(npix,1)=ilt 
      goto,lt_end 
    endif
  endfor
  lt_end: 
  exist_tile(ilg,ilt)=1  
endfor


file20=outdir+'tile_number.res'
openw,20,file20

tilenum = -1

for ilt=0,5 do begin
 for ilg=0,11 do begin
  tilenum = tilenum + 1
  if exist_tile(ilg,ilt) eq 1 then begin 
   printf,20,tilenum,format='(i2)'     
  endif  
 endfor
endfor

close,10
close,20

tilenum=-1

for ilt=0,5 do begin
 for ilg=0,11 do begin
  tilenum = tilenum + 1
  if exist_tile(ilg,ilt) eq 1 then begin 
   if tilenum le 9 then begin  
    tile=string(tilenum,format='(i1)',/print)
   endif else begin
    tile=string(tilenum,format='(i2)',/print)
   endelse
   print,'Tile: ',tile
   latmin=-90.+ilt*30.
   latmax=latmin+30.
   longmin=ilg*30.-180.
   longmax=longmin+30.
   var='alt'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_alt
   var='tavg'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_tavg
   var='tmax'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_tmax
   var='tmin'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_tmin 
   var='prec'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_prec
   var='srad'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_srad
   var='vapr'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_vapr
   var='wind'
   read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_wind                 
;   outform='f5.1'
;   write_tile,tile,var,latmin,longmin,outdir,outform,xlong,xlat,var_tavg

   for npix = long(0), npix_max-long(1) do begin
     if tile_pix(npix,0) eq ilg then begin 
      if tile_pix(npix,1) eq ilt then begin       
;        print,xlong_pix(npix),xlat_pix(npix)
	ilg_pix=fix((xlong_pix(npix)-longmin)*120.)
	ilt_pix=fix((latmax-xlat_pix(npix))*120.)
	if (var_alt(ilg_pix,ilt_pix,0) gt elvthr) and (var_tavg(ilg_pix,ilt_pix,0) gt -900.) then begin
	 alt_pix(npix)=var_alt(ilg_pix,ilt_pix,0)
	 tavg_pix(npix,*)=var_tavg(ilg_pix,ilt_pix,*)
	 delt_pix(npix,*)=var_tmax(ilg_pix,ilt_pix,*)-var_tmin(ilg_pix,ilt_pix,*)
	 for m=0,11 do begin
	  if delt_pix(npix,m) lt 0. then begin
	   print,'Tmax smaller than Tmin: ',tile,npix,m
	   print,'    ',var_tmax(ilg_pix,ilt_pix,m),var_tmin(ilg_pix,ilt_pix,m) 
	   delt_pix(npix)=0.
	  endif
	 endfor
	 prec_pix(npix,*)=var_prec(ilg_pix,ilt_pix,*)
	 srad_pix(npix,*)=var_srad(ilg_pix,ilt_pix,*)
	 vapr_pix(npix,*)=var_vapr(ilg_pix,ilt_pix,*)
	 wind_pix(npix,*)=var_wind(ilg_pix,ilt_pix,*)
;         print,ilg_pix,ilt_pix
         for m = 0, 11 do begin
	   tcel = tavg_pix(npix,m)
; svp = saturated vapour pressure (hPa)	   
	   svp = 6.11*(10.^(7.5*tcel/(237.7+tcel)))
           rh= 100.*vapr_pix(npix,m)/svp
	   if rh lt 5. then begin
	    print,'RH too small: ',rh,xlong_pix(npix),xlat_pix(npix),m
	    rh = 5.
	   endif else begin
	    if rh gt 100. then begin
	     print,'RH too large: ',rh,xlong_pix(npix),xlat_pix(npix),m
	     rh = 100.	  
	    endif
	   endelse
	   rhu_pix(npix,m)=rh
	 endfor   
        endif else begin
         alt_pix(npix)=-999.
	 tavg_pix(npix,*)=-99.9
	 delt_pix(npix,*)=-99.9
	 prec_pix(npix,*)=-99.9
	 srad_pix(npix,*)=-99.9
         vapr_pix(npix,*)=-99.9
         wind_pix(npix,*)=-99.9
	 rhu_pix(npix,*)=-99.9	 
        endelse
      endif    
     endif
   endfor     
  endif  
 endfor
endfor

if kwrite ne 0 then begin
 close,31
 file31=outdir+'elev_WC.dat'
 openw,31,file31

 if kwrite eq 1 then begin
  close,32
  file32=outdir+'tem_WC.dat'
  openw,32,file32

  close,33
  file33=outdir+'dte_WC.dat'
  openw,33,file33

  close,34
  file34=outdir+'prc_WC.dat'
  openw,34,file34
 
  close,35
  file35=outdir+'fsol_WC.dat'
  openw,35,file35
 
  close,36
  file36=outdir+'rhu_WC.dat'
  openw,36,file36
 
  close,37
  file37=outdir+'wnd_WC.dat'
  openw,37,file37   
 
  if (ifilform eq 1) then begin
   close,38
   file38=outdir+'species_WC.dat'
   openw,38,file38
  endif
 endif

 for npix = long(0), npix_max-long(1) do begin
  if (alt_pix(npix) gt -900.) and (tavg_pix(npix,0) gt -99.) then begin
   if kwrite eq 1 then begin  
    if (delt_pix(npix,0) gt -99.) and (prec_pix(npix,0) gt -99.) then begin
     if (srad_pix(npix,0) gt -99.) and (rhu_pix(npix,0) gt -99.) and (wind_pix(npix,0) gt -99.) then begin
      printf,31,xlong_pix(npix),xlat_pix(npix),alt_pix(npix),format='(1x,f9.4,1x,f8.4,1x,f5.0)'   
      printf,32,xlong_pix(npix),xlat_pix(npix),tavg_pix(npix,*),format='(1x,f9.4,1x,f8.4,12(1x,f5.1))'
      printf,33,xlong_pix(npix),xlat_pix(npix),delt_pix(npix,*),format='(1x,f9.4,1x,f8.4,12(1x,f5.1))'
      printf,34,xlong_pix(npix),xlat_pix(npix),prec_pix(npix,*),format='(1x,f9.4,1x,f8.4,12(1x,f5.0))'
      printf,35,xlong_pix(npix),xlat_pix(npix),srad_pix(npix,*),format='(1x,f9.4,1x,f8.4,12(1x,f6.1))'
      printf,36,xlong_pix(npix),xlat_pix(npix),rhu_pix(npix,*),format='(1x,f9.4,1x,f8.4,12(1x,f6.2))'
      printf,37,xlong_pix(npix),xlat_pix(npix),wind_pix(npix,*),format='(1x,f9.4,1x,f8.4,12(1x,f5.1))'         
      if ifilform eq 1 then printf,38,xlong_pix(npix),xlat_pix(npix),ioc_pix(npix,*),format='(1x,f9.4,1x,f8.4,200(1x,i2))'
     endif
    endif
   endif else begin
    printf,31,xlong_pix(npix),xlat_pix(npix),alt_pix(npix),format='(1x,f9.4,1x,f8.4,1x,f5.0)'  
   endelse
  endif
 endfor
endif

close,/all

;for npix = long(0), npix_max-long(1) do begin
;  slt=string(tile_pix(npix,1),format='(i1)',/print)
;  if tile_pix(npix,0) le 9 then begin 
;    slg=string(tile_pix(npix,0),format='(i1)',/print) 
;    printf,20,slt,slg,format='(a1,a1)'
;  endif else begin
;    slg=string(tile_pix(npix,0),format='(i2)',/print)
;    printf,20,slt,slg,format='(a1,a2)'
;  endelse 
;endfor

if kmap eq 1 then begin

 if ifile eq 0 then begin
   dlong_m = dlong
   dlat_m = dlat
 endif else begin
   dlong_m = dlong*pix_amplify
   dlat_m = dlat*pix_amplify
 endelse

 if kwrite eq 2 then begin
  device, true=24,retain=2,decomposed=0
  !p.multi=[0,1,1]
  !p.background=255
  !p.color=0  
  goto,elevation
 endif
  
 nextplot:
 print,'Which month do you want to plot ?'
 print,'-1 = Exit'
 print,'0 = Annual'
 print,'1 = January'
 print,'2 = February'
 print,'...'
 print,'12 = December'
 read,im
 
 if im le -1 or im ge 13 then goto,finish

  device, true=24,retain=2,decomposed=0
 !p.multi=[0,2,2]
 !p.background=255
 !p.color=0
 
 erase
 
; win = !p.multi(0)   


 tavg_V=fltarr(npix_max)
 delt_V=fltarr(npix_max)
 prec_V=fltarr(npix_max)
 srad_V=fltarr(npix_max)
 rhu_V=fltarr(npix_max)
 wind_V=fltarr(npix_max)
 if im eq 0 then begin   
   for npix = long(0), npix_max-long(1) do begin
     tavg_V(npix) = mean(tavg_pix(npix,*))
     delt_V(npix) = mean(delt_pix(npix,*))
     prec_V(npix) = total(prec_pix(npix,*))
     srad_V(npix) = mean(srad_pix(npix,*))
     rhu_V(npix) = mean(rhu_pix(npix,*))
     wind_V(npix) = mean(wind_pix(npix,*))
   endfor
   labmth='Annual'
 endif else begin
   tavg_V(*)=tavg_pix(*,im-1)
   delt_V(*)=delt_pix(*,im-1)              
   prec_V(*)=prec_pix(*,im-1)
   srad_V(*)=srad_pix(*,im-1)
   rhu_V(*)=rhu_pix(*,im-1)
   wind_V(*)=wind_pix(*,im-1)
   labmth='Month '+string(im,format='(i2)',/print)
 endelse 
 
 erase

 elevation: 

 win = 0
 ipal = 0
 idir = 1
 color_scale_10,r,g,b,ipal,idir 
    
 map_title=' Elevation (m)'
 xtic_dat = [-50,-20,0,50,100,200,300,400,600,800,1000,1200,1400,1600,1800,2000,2500,3000,4000,5000]
 WC_plot,xlong_pix,xlat_pix,alt_pix,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title
 
 if kwrite eq 2 then goto,finish 
 
 ;wait,5.0
 
 win = 3

 ipal = 1
 idir = 1
 color_scale_10,r,g,b,ipal,idir 
    
 map_title=labmth+' Temperature (!eo!nC)'
 var_V = tavg_V
 xtic_dat = [-99,-15,-10,-5,0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30]
 WC_plot,xlong_pix,xlat_pix,var_V,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title 
 
 ;wait,5.0  
 
 win = 2
 
 map_title=labmth+' Tmax-Tmin (!eo!nC)'
 var_V = delt_V
 xtic_dat = [0,1,2,3,4,5,6,8,10,12,14,16,18,20,22,24,26,28,30]
 WC_plot,xlong_pix,xlat_pix,var_V,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title
 
 ;wait,5.0
 
 win = 1
 
 ipal = 1
 idir = -1
 color_scale_10,r,g,b,ipal,idir 

 map_title=labmth+' Precipitation (mm)'
 var_V = prec_V
 if im eq 0 then begin
  xtic_dat = [0,20,50,100,150,200,300,400,500,600,800,1000,1200,1400,1600,1800,2000,2500,3000,4000]  
 endif else begin 
  xtic_dat = [0,5,10,15,20,25,30,40,50,60,80,100,120,140,160,180,200,250,300,400]
 endelse
 WC_plot,xlong_pix,xlat_pix,var_V,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title 

   print,' Press <return> key to continue '
   answer = ''
   read,answer 
;  wait,10.0 
  erase
  
 win = 0
 ipal = 0
 idir = 1
 color_scale_10,r,g,b,ipal,idir 
    
 map_title=' Elevation (m)'
 xtic_dat = [-50,-20,0,50,100,200,300,400,600,800,1000,1200,1400,1600,1800,2000,2500,3000,4000,5000]
 WC_plot,xlong_pix,xlat_pix,alt_pix,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title 
 
 ;wait,5.0
 
 win = 3

 ipal = 1
 idir = 1
 color_scale_10,r,g,b,ipal,idir 
    
 map_title=labmth+' Solar Radiation (W m!e-2!n)'
 var_V = srad_V
 xtic_dat = [-99,0,20,40,60,80,100,120,140,160,180,200,220,240,260,280,300,320,340,360]
 WC_plot,xlong_pix,xlat_pix,var_V,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title 
 
 ;wait,5.0 
  
 win = 2

 ipal = 1
 idir = -1
 color_scale_10,r,g,b,ipal,idir 
    
 map_title=labmth+' Air Relative Humidity (%)'
 var_V = rhu_V
 xtic_dat = [5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,100]
 WC_plot,xlong_pix,xlat_pix,var_V,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title 
 
 ;wait,5.0  
 
 win = 1

 ipal = 1
 idir = 1
 color_scale_10,r,g,b,ipal,idir 
    
 map_title=labmth+' Wind Speed (m s!e-1!n)'
 var_V = wind_V
 xtic_dat = [0,0.5,1,1.5,2,2.5,3,3.5,4,5,6,7,8,9,10,12,14,16,18,20]
 WC_plot,xlong_pix,xlat_pix,var_V,win,dlong_m,dlat_m,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title 
 
 ;wait,5.0   
  
 
;  stop
; this line stops the execution of the programme, use ".cont" to continue  
 
goto,nextplot 
 

endif

finish:

print,'End of program'

return
end 

;====================================================================================

pro read_WC2,tile,var,latmin,longmin,latmax,longmax,xlong,xlat,var_mth

dlat=1./120.
dlong=1./120.

maindir='/raid3/umccb/WorldClim2/data/'


close,1
elev=intarr(3600,3600)
xlat=fltarr(3600)
xlong=fltarr(3600)

for ilat=1,3600 do xlat(ilat-1)=latmax-dlat*(0.5+float(ilat-1))
for ilong=1,3600 do xlong(ilong-1)=longmin+dlong*(0.5+float(ilong-1))

print,'Reading file(s)'

if var eq 'alt' then begin
   nm = 1
   var_mth=fltarr(3600,3600,nm)
   file=maindir+var+'/'+var+'_'+tile+'.bil'
   print,'Reading: ',file
   openr,1,file
   readu,1,elev
   var_mth(*,*,0)=float(elev(*,*))
   close,1
endif else begin
   nm = 12
   var_mth=fltarr(3600,3600,nm)
   scfac = 1.
;   if var eq 'prec' then scfac=1.
;   if var eq 'tavg' then scfac=0.1
;   if var eq 'tmin' then scfac=0.1
;   if var eq 'tmax' then scfac=0.1 
; solar radiation: conversion from kJ m-2 day-1 to W m -2
   if var eq 'srad' then scfac=1.e+3/86400.  
; vapour pressure: conversion from kPa to hPa
   if var eq 'vapr' then scfac=10.   
   for m = 1, nm do begin
     if m le 9 then mth=string('0',m,format='(a1,i1)',/print) else mth=string(m,format='(i2)',/print)
     file=maindir+var+'/wc2.0_30s_'+var+'_'+mth+'_'+tile+'.TIF'
     print,'Reading: ',file
     vari=read_tiff(file,orientation=ori)
     sz=size(vari)
;     print,'size: ',sz
;     print,'ori: ',ori
     nlg = sz(1)
     nlt = sz(2)
     m_ind=m-1     
     var_mth(*,*,m_ind)=scfac*float(vari(*,*))
   endfor
endelse


return
end
     
;===================================================================================
pro write_tile,tile,var,latmin,longmin,outdir,outform,xlong,xlat,var_mth

outfile=outdir+var+'_'+tile+'.txt'
openw,2,outfile
print,'Writing file: ',outfile

form='(2(1x,f10.5),12(1x,'+outform+'))'

dlat=1./120.
dlong=1./120.
xlat=fltarr(3600)
xlong=fltarr(3600)

for ilat=1,3600 do xlat(ilat-1)=latmin+dlat*(0.5+float(ilat-1))
for ilong=1,3600 do xlong(ilong-1)=longmin+dlong*(0.5+float(ilong-1))

for ilat=1,3600 do begin
 for ilong=1,3600 do begin
  if var_mth(ilong-1,ilat-1,0) gt -900. then begin
    printf,2,xlong(ilong-1),xlat(ilat-1),var_mth(ilong-1,ilat-1,*),format=form
  endif
 endfor
endfor

close,2

return
end

;================
pro WC_plot,lg,lat,var_V,win,dlg,dlt,lgmin,ltmin,lgmax,ltmax,xtic_dat,map_title

 resol=fltarr(2)
 resol(0)=dlg
 resol(1)=dlt

 print,map_title
 v_min=min(var_V)
 print,'  Min value:',v_min
 v_max=max(var_V)
 print,'  Max value:',v_max
 resol=fltarr(2)
 resol(0)=dlg
 resol(1)=dlt

 format_ticlab=['i6','   ',' - ',' > ']
 top_margin = 4 
 
; !p.multi(0)=!p.multi(0)-1
 map_plot,lg,lat,var_V,win,MAP_TITLE=map_title,MAPLIM=[lgmin,ltmin,lgmax,ltmax],RESOL=resol,PROJECTION_NAME='Cylindrical',XTIC_DAT=xtic_dat,FORMAT_TICLAB=format_ticlab,TOP_MARGIN=top_margin,/igrid,/icont
    
return
end 

